
/**************************************************************************
**  (c) Copyright 2002, Andromeda Technology & Automation
** This is free software; you can redistribute it and/or modify it under the
** terms of the GNU General Public License, see the file COPYING.
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : gnucomo_config.cpp
**      SYSTEM NAME    : Gnucomo - Gnu Computer Monitoring
**      VERSION NUMBER : $Revision: 1.4 $
**
**  DESCRIPTION      :  Implementation of the gnucomo_config class.
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Jul 24, 2002
**      LAST UPDATE     : Nov 02, 2002
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: gnucomo_config.cpp,v $
   Revision 1.4  2002/12/06 22:32:11  arjen
   Added new arguments to gnucomo_config::Database(): user and password.
   If empty, default values are taken from the config file.

   Revision 1.3  2002/11/09 08:04:27  arjen
   Added a reference to the GPL

   Revision 1.2  2002/11/04 10:13:36  arjen
   Use proper namespace for iostream classes

   Revision 1.1  2002/10/05 10:25:49  arjen
   Creation of gcm_input and a first approach to a web interface

*****************************/

static const char *RCSID = "$Id: gnucomo_config.cpp,v 1.4 2002/12/06 22:32:11 arjen Exp $";

#include "gnucomo_config.h"



/*=========================================================================
**  NAME           : Database
**  SYNOPSIS       : String gnucomo_config::Database()
**  PARAMETERS     : 
**  RETURN VALUE   : The database access string
**
**  DESCRIPTION    : If the parameters 'usr' and 'pw' are empty, the
**                   username and password from the configuration file
**                   are used.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Nov 21, 2002
**=========================================================================
*/

String gnucomo_config::Database(String usr, String pw)
{
   String param;
   String access_string("");

   param = find_parameter("database", "name");
   if (param != "")
   {
      access_string += "dbname=" + param;
   }

   if (usr == "")
   {
      param = find_parameter("database", "user");
      if (param != "")
      {
         access_string += " user=" + param;
      }
   }
   else
   {
      access_string += " user=" + usr;
   }

   if (pw == "")
   {
      param = find_parameter("database", "password");
      if (param != "")
      {
         access_string += " password=" + param;
      }
   }
   else
   {
      access_string += " password=" + pw;
   }

   param = find_parameter("database", "host");
   if (param != "")
   {
      access_string += " host=" + param;
   }
   param = find_parameter("database", "port");
   if (param != "")
   {
      access_string += " port=" + param;
   }
   return access_string;
}
